/*
 Realisiert mit Tiny25
 
 Interner Oszillator mit 1MHz, 
 (Vorteiler /8 eingeschaltet)
 
 Layout:
                      Reset  1    8  +5V
                 IR-Led PB3  2     7  PB2 Ubatt/2
                        PB4  3     6  PB1 Bereit-Led Rot
                        GND  4     5  PB0 Bereit-Led Grn

 Input: 
   PB2  halbierte Batteriespannung (vor dem Spannungsregler)
 Output:
   PB0  Ausgang zur Zweifarb-LED, Anode 
   PB1  Ausgang zur Zweifarb-LED, Kathode 
   PB3  Ausgang zur IR-Sende-LED  LD271
*/

/*
   Sendet Bursts von 12 Pulsen mit 38KHz. Dazwischen 16 Pulse Pause.
   Der Empfnger braucht 6-8 Pulse zum einschwingen und mind. 12 Pulse Pause.
   Ein Durchlauf dauert also 28 Pulse. Bei 28.3s Zykluszeit sind das 792.4s,
   entsprechend 1262Hz. Damit ist eine Messung mit Millisekundenauflsung mglich.
   
   Die Batteriespannung wird zyklisch gemessen und auf einer Duo-LED 
   der Batteriestatus in grn oder rot blitzend angezeigt.
   Statt Duo-LED knnen auch 2 antiparallel geschaltete LEDs verwendet werden.
*/

#define F_CPU 1000000UL	/* CPU clock in Hertz */

#include <avr/io.h>
#include <util/delay.h>

#define LED_ROT {PORTB&=~BIT0; PORTB|=BIT1;}
#define LED_GRUEN {PORTB&=~BIT1; PORTB|=BIT0;}
#define LED_AUS {PORTB&=~BIT0; PORTB&=~BIT1;}
#define IRLED_AUS {PORTB&=~BIT3;}
#define IRLED_EIN {PORTB|=BIT3;}

#define U8 unsigned char
#define BIT0 1
#define BIT1 2
#define BIT2 4
#define BIT3 8
#define BIT4 16


int main(void){
  U8 n;
  U8 status=0;
  U8 zyklus=1;
  U8 Ubatt;

  DDRB=BIT0|BIT1|BIT3|BIT4;
  PORTB=0;
  ADMUX=(1<<ADLAR) | 1; //Uref=5V
  ADCSRA=(1<<ADEN) | (1<<ADSC)  | (1<<ADPS2) | (1<<ADPS1); //125KHz
  ACSR=(1<<ACD); //Analogkomparator aus
  PRR=(1<<PRTIM1)|(1<<PRTIM0)|(1<<PRUSI);

  while(1){
    //Infrarotsender: 38KHz Traegerfrequenz (26.3s Zykluszeit)
    for (n=0; n<12; n++){ //12 Pulse Signal
      IRLED_EIN;
      _delay_us(8); //Bei 1MHz ist der Delay nicht so genau. Handgetrimmt.
//      _delay_us(13);
      IRLED_AUS;
      _delay_us(8);
//      _delay_us(13);
    }

    //16 Pulse Pause
    _delay_us(26*16);

    // Hier komme ich alle 728s hin (1,37KHz)
    
    //Batteriemessung
    switch(status){
      case 0:  //ca. 5 mal pro Sekunde
        ADCSRA|=(1<<ADSC); //Alle 186ms
        break;

      case 2: //nach 1,4ms ist das Ergebnis sicher da
        Ubatt=ADCH; //8V=186 9V=211
        if (!--zyklus){
          if (Ubatt>=186){
            LED_GRUEN;
            if (Ubatt>201){ //8,6V
              zyklus=3;
            }else{
              zyklus=11;
            }
          }else{
            LED_ROT;
            zyklus=6;
          }
        }
        break;

      case 36:  //Led nach 25ms wieder aus
        LED_AUS;
        break;
    }//switch
    status++; //luft bei 255 ber auf 0

  }
}
